@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2015 Oracle and/or its affiliates. All rights reserved.
+ *     
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -45,9 +43,8 @@ const unsigned int FunctionNamespaceURIFromQName::maxArgs = 1;
 **/
 
 FunctionNamespaceURIFromQName::FunctionNamespaceURIFromQName(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : ConstantFoldingFunction(name, minArgs, maxArgs, "QName?", args, memMgr)
+  : XQFunction(name, "($arg as xs:QName?) as xs:anyURI?", args, memMgr)
 {
-  _src.getStaticType() = StaticType(StaticType::ANY_URI_TYPE, 0, 1);
 }
 
 Sequence FunctionNamespaceURIFromQName::createSequence(DynamicContext* context, int flags) const
