@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2015 Oracle and/or its affiliates. All rights reserved.
+ *     
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
@@ -39,26 +37,20 @@ const unsigned int FunctionTrace::maxArgs = 2;
  */
 
 FunctionTrace::FunctionTrace(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr)
-  : XQFunction(name, minArgs, maxArgs, "item()*, string", args, memMgr)
+  : XQFunction(name, "($value as item()*, $label as xs:string) as item()*", args, memMgr)
 {
 }
 
-ASTNode* FunctionTrace::staticResolution(StaticContext *context) {
-  return resolveArguments(context);
-}
-
 ASTNode *FunctionTrace::staticTypingImpl(StaticContext *context)
 {
-  _src.clear();
+  _src.clearExceptType();
+  calculateSRCForArguments(context);
 
-  ASTNode *result = calculateSRCForArguments(context);
-  if(result == this) {
-    _src.getStaticType() = _args.front()->getStaticAnalysis().getStaticType();
-    _src.setProperties(_args.front()->getStaticAnalysis().getProperties());
-  }
+  _src.getStaticType() = _args.front()->getStaticAnalysis().getStaticType();
+  _src.setProperties(_args.front()->getStaticAnalysis().getProperties());
   _src.forceNoFolding(true);
 
-  return result;
+  return this;
 }
 
 Sequence FunctionTrace::createSequence(DynamicContext* context, int flags) const
