@@ -16,14 +16,13 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * $Id: XQUserFunction.cpp 770 2009-06-05 15:26:55Z jpcs $
+ * $Id$
  */
 
 #include <sstream>
 
 #include <xqilla/framework/XQillaExport.hpp>
 #include <xqilla/functions/XQUserFunction.hpp>
-#include <xqilla/functions/FunctionConstructor.hpp>
 #include <xqilla/exceptions/FunctionException.hpp>
 #include <xqilla/exceptions/XPath2TypeMatchException.hpp>
 #include <xqilla/exceptions/StaticErrorException.hpp>
@@ -40,6 +39,9 @@
 #include <xqilla/exceptions/StaticErrorException.hpp>
 #include <xqilla/runtime/ClosureResult.hpp>
 #include <xqilla/optimizer/ASTVisitor.hpp>
+#include <xqilla/optimizer/StaticTyper.hpp>
+
+#include "../lexer/XQLexer.hpp"
 
 #include <xercesc/util/XMLString.hpp>
 #include <xercesc/framework/XMLBuffer.hpp>
@@ -70,47 +72,44 @@ const XMLCh XQUserFunction::XMLChXQueryLocalFunctionsURI[] =
     chNull
 };
 
-XQUserFunction::XQUserFunction(const XMLCh *qname, ArgumentSpecs *argSpecs, ASTNode *body,
-                               SequenceType *returnType, bool isUpdating, bool isGlobal,
-                               XPath2MemoryManager *mm)
-  : FuncFactory(argSpecs == 0 ? 0 : argSpecs->size(), mm),
+XQUserFunction::XQUserFunction(const XMLCh *qname, FunctionSignature *signature,
+                               ASTNode *body, bool isGlobal, XPath2MemoryManager *mm)
+  : FuncFactory(signature->argSpecs == 0 ? 0 : signature->argSpecs->size(), mm),
     body_(body),
     exFunc_(NULL),
     qname_(qname),
     pattern_(0),
     templateInstance_(0),
     modes_(0),
-    returnType_(returnType),
-    argSpecs_(argSpecs),
+    signature_(signature),
     isGlobal_(isGlobal),
-    isUpdating_(isUpdating),
     isTemplate_(false),
     memMgr_(mm),
     src_(mm),
-    staticTyped_(false),
+    staticTyped_(BEFORE),
     recursive_(false),
+    delayed_(false),
     moduleDocCache_(NULL)
 {
 }
 
-XQUserFunction::XQUserFunction(const XMLCh *qname, VectorOfASTNodes *pattern, ArgumentSpecs *argSpecs,
-                               ASTNode *body, SequenceType *returnType, XPath2MemoryManager *mm)
-  : FuncFactory(argSpecs == 0 ? 0 : argSpecs->size(), mm),
+XQUserFunction::XQUserFunction(const XMLCh *qname, VectorOfASTNodes *pattern, FunctionSignature *signature,
+                               ASTNode *body, XPath2MemoryManager *mm)
+  : FuncFactory(signature->argSpecs == 0 ? 0 : signature->argSpecs->size(), mm),
     body_(body),
     exFunc_(NULL),
     qname_(qname),
     pattern_(pattern),
     templateInstance_(0),
     modes_(0),
-    returnType_(returnType),
-    argSpecs_(argSpecs),
+    signature_(signature),
     isGlobal_(true),
-    isUpdating_(false),
     isTemplate_(true),
     memMgr_(mm),
     src_(mm),
-    staticTyped_(false),
+    staticTyped_(BEFORE),
     recursive_(false),
+    delayed_(false),
     moduleDocCache_(NULL)
 {
 }
@@ -123,15 +122,14 @@ XQUserFunction::XQUserFunction(const XQUserFunction *o, XPath2MemoryManager *mm)
     pattern_(0),
     templateInstance_(o->templateInstance_),
     modes_(0),
-    returnType_(o->returnType_),
-    argSpecs_(0),
+    signature_(new (mm) FunctionSignature(o->signature_, mm)),
     isGlobal_(o->isGlobal_),
-    isUpdating_(o->isUpdating_),
     isTemplate_(o->isTemplate_),
     memMgr_(mm),
     src_(mm),
     staticTyped_(o->staticTyped_),
     recursive_(o->recursive_),
+    delayed_(o->delayed_),
     moduleDocCache_(o->moduleDocCache_)
 {
   if(o->pattern_) {
@@ -146,14 +144,6 @@ XQUserFunction::XQUserFunction(const XQUserFunction *o, XPath2MemoryManager *mm)
       modes_->push_back(new (mm) Mode(*modeIt));
     }
   }
-  if(o->argSpecs_) {
-    argSpecs_ = new (mm) ArgumentSpecs(XQillaAllocator<ArgumentSpec*>(mm));
-
-    XQUserFunction::ArgumentSpecs::const_iterator argIt = o->getArgumentSpecs()->begin();
-    for(; argIt != o->getArgumentSpecs()->end(); ++argIt) {
-      argSpecs_->push_back(new (mm) ArgumentSpec(*argIt, mm));
-    }
-  }
   setLocationInfo(o);
   src_.copy(o->src_);
 }
@@ -179,19 +169,7 @@ void XQUserFunction::releaseImpl()
     memMgr_->deallocate(modes_);
   }
 
-  if(argSpecs_) {
-    XQUserFunction::ArgumentSpecs::iterator argIt = argSpecs_->begin();
-    for(; argIt != argSpecs_->end(); ++argIt) {
-      const_cast<StaticAnalysis&>((*argIt)->getStaticAnalysis()).clear();
-      memMgr_->deallocate(*argIt);
-    }
-#if defined(_MSC_VER) && (_MSC_VER < 1300)
-    argSpecs_->~vector<ArgumentSpec*,XQillaAllocator<ArgumentSpec*> >();
-#else
-    argSpecs_->~ArgumentSpecs();
-#endif
-    memMgr_->deallocate(argSpecs_);
-  }
+  signature_->release();
 
   src_.clear();
   memMgr_->deallocate(this);
@@ -202,26 +180,6 @@ ASTNode* XQUserFunction::createInstance(const VectorOfASTNodes &args, XPath2Memo
   return new (mm) XQUserFunctionInstance(this, args, mm);
 }
 
-void XQUserFunction::ArgumentSpec::staticResolution(StaticContext* context)
-{
-  if(qname_ != 0) {
-    uri_ = context->getUriBoundToPrefix(XPath2NSUtils::getPrefix(qname_, context->getMemoryManager()), this);
-    name_ = XPath2NSUtils::getLocalName(qname_);
-  }
-
-  seqType_->staticResolution(context);
-
-  bool isPrimitive;
-  seqType_->getStaticType(src_.getStaticType(), context, isPrimitive, seqType_);
-
-  if(seqType_->getOccurrenceIndicator() == SequenceType::EXACTLY_ONE ||
-     seqType_->getOccurrenceIndicator() == SequenceType::QUESTION_MARK) {
-    src_.setProperties(StaticAnalysis::DOCORDER | StaticAnalysis::GROUPED |
-                       StaticAnalysis::PEER | StaticAnalysis::SUBTREE | StaticAnalysis::SAMEDOC |
-                       StaticAnalysis::ONENODE | StaticAnalysis::SELF);
-  }
-}
-
 void XQUserFunction::Mode::staticResolution(StaticContext* context)
 {
   if(qname_ != 0) {
@@ -239,11 +197,11 @@ bool XQUserFunction::Mode::equals(const Mode *o) const
     XPath2Utils::equals(name_, o->name_);
 }
 
-void XQUserFunction::staticResolutionStage1(StaticContext *context)
+void XQUserFunction::resolveName(StaticContext *context)
 {
   XPath2MemoryManager *mm = context->getMemoryManager();
 
-  if(qname_ != 0) {
+  if(qname_ != 0 && name_ == 0) {
     const XMLCh *prefix = XPath2NSUtils::getPrefix(qname_, mm);
     name_ = XPath2NSUtils::getLocalName(qname_);
 
@@ -257,26 +215,32 @@ void XQUserFunction::staticResolutionStage1(StaticContext *context)
 
   if(name_ != 0) {
     setURINameHash(uri_, name_);
+  }
+}
 
-    if(!isTemplate_) {
-      if(XPath2Utils::equals(uri_, XMLUni::fgXMLURIName) ||
-         XPath2Utils::equals(uri_, SchemaSymbols::fgURI_SCHEMAFORSCHEMA) ||
-         XPath2Utils::equals(uri_, SchemaSymbols::fgURI_XSI) ||
-         XPath2Utils::equals(uri_, XQFunction::XMLChFunctionURI) ||
-         XPath2Utils::equals(uri_, FunctionConstructor::XMLChXPath2DatatypesURI)) {
-        XQThrow(FunctionException, X("XQUserFunction::staticResolutionStage1"),
-                X("A user defined function must not be in the namespaces xml, xsd, xsi, fn or xdt [err:XQST0045]"));
-      }
-      else if(uri_ == 0 || *uri_ == 0)
-        XQThrow(FunctionException, X("XQUserFunction::staticResolutionStage1"),
-                X("A user defined function must be defined in a namespace [err:XQST0060]"));
+void XQUserFunction::staticResolutionStage1(StaticContext *context)
+{
+  XPath2MemoryManager *mm = context->getMemoryManager();
+
+  resolveName(context);
+
+  if(name_ != 0 && !isTemplate_ && !delayed_) {
+    if(XPath2Utils::equals(uri_, XMLUni::fgXMLURIName) ||
+       XPath2Utils::equals(uri_, SchemaSymbols::fgURI_SCHEMAFORSCHEMA) ||
+       XPath2Utils::equals(uri_, SchemaSymbols::fgURI_XSI) ||
+       XPath2Utils::equals(uri_, XQFunction::XMLChFunctionURI) ||
+       XPath2Utils::equals(uri_, SchemaSymbols::fgURI_SCHEMAFORSCHEMA)) {
+      XQThrow(FunctionException, X("XQUserFunction::staticResolutionStage1"),
+              X("A user defined function must not be in the namespaces xml, xsd, xsi, fn or xdt [err:XQST0045]"));
     }
+    else if(uri_ == 0 || *uri_ == 0)
+      XQThrow(FunctionException, X("XQUserFunction::staticResolutionStage1"),
+              X("A user defined function must be defined in a namespace [err:XQST0060]"));
   }
 
   // Check for the implementation of an external function
   if(body_ == NULL) {
-    size_t nArgs = argSpecs_ ? argSpecs_->size() : 0;
-    exFunc_ = context->lookUpExternalFunction(uri_, name_, nArgs);
+    exFunc_ = context->lookUpExternalFunction(uri_, name_, minArgs_);
 
     if(exFunc_ == NULL) {
       XMLBuffer buf;
@@ -286,48 +250,14 @@ void XQUserFunction::staticResolutionStage1(StaticContext *context)
       buf.append(name_);
       buf.append(X("' with "));
       XMLCh szNumBuff[20];
-      XMLString::binToText((unsigned int)nArgs, szNumBuff, 19, 10);
+      XMLString::binToText((unsigned int)minArgs_, szNumBuff, 19, 10);
       buf.append(szNumBuff);
       buf.append(X(" argument(s) has not been bound to an implementation"));
       XQThrow(FunctionException, X("XQUserFunction::staticResolutionStage1"), buf.getRawBuffer());
     }
   }
 
-  // Resolve the parameter names
-  if(argSpecs_) {
-    ArgumentSpecs::iterator it;
-    for(it = argSpecs_->begin(); it != argSpecs_->end (); ++it) {
-      (*it)->staticResolution(context);
-    }
-    // check for duplicate parameters
-    for(it = argSpecs_->begin(); argSpecs_->size() > 1 && it != argSpecs_->end()-1; ++it) {
-      for(ArgumentSpecs::iterator it2 = it+1; it2 != argSpecs_->end (); ++it2) {
-        if(XPath2Utils::equals((*it)->getURI(),(*it2)->getURI()) && 
-           XPath2Utils::equals((*it)->getName(),(*it2)->getName())) {
-          XMLBuffer buf;
-          if(isTemplate_) {
-            buf.set(X("Template "));
-          } else {
-            buf.set(X("User-defined function "));
-          }
-          if(name_ != 0) {
-            buf.append(X("'{"));
-            buf.append(uri_);
-            buf.append(X("}"));
-            buf.append(name_);
-            buf.append(X("' "));
-          }
-          buf.append(X("has two parameters with the expanded QName '"));
-          buf.append(X("{"));
-          buf.append((*it)->getURI());
-          buf.append(X("}"));
-          buf.append((*it)->getName());
-          buf.append(X("' [err:XQST0039]"));
-          XQThrow(StaticErrorException, X("XQUserFunction::staticResolution"), buf.getRawBuffer());
-        }
-      }
-    }
-  }
+  signature_->staticResolution(context);
 
   // Resolve the mode names
   if(modes_) {
@@ -365,22 +295,20 @@ void XQUserFunction::staticResolutionStage1(StaticContext *context)
   }
 
   // Set up a default StaticType and StaticAnalysis
-  if(returnType_ != NULL) {
-    returnType_->staticResolution(context);
-
+  if(signature_->returnType) {
     if(body_ != NULL) {
-      body_ = returnType_->convertFunctionArg(body_, context, /*numericfunction*/false, returnType_);
+      body_ = signature_->returnType->convertFunctionArg(body_, context, /*numericfunction*/false, signature_->returnType);
     }
 
     bool isPrimitive;
-    returnType_->getStaticType(src_.getStaticType(), context, isPrimitive, returnType_);
+    signature_->returnType->getStaticType(src_.getStaticType(), context, isPrimitive, signature_->returnType);
   }
   else {
     // Default type is item()*
     src_.getStaticType() = StaticType(StaticType::ITEM_TYPE, 0, StaticType::UNLIMITED);
   }
 
-  if(isUpdating_) {
+  if(signature_->updating == FunctionSignature::OP_TRUE) {
     src_.updating(true);
   }
 
@@ -399,9 +327,9 @@ void XQUserFunction::staticResolutionStage1(StaticContext *context)
     // Build an instance of the template for us to call
     VectorOfASTNodes newArgs = VectorOfASTNodes(XQillaAllocator<ASTNode*>(mm));
 
-    if(argSpecs_ != 0) {
-      XQUserFunction::ArgumentSpecs::const_iterator argIt;
-      for(argIt = argSpecs_->begin(); argIt != argSpecs_->end(); ++argIt) {
+    if(signature_->argSpecs != 0) {
+      ArgumentSpecs::const_iterator argIt;
+      for(argIt = signature_->argSpecs->begin(); argIt != signature_->argSpecs->end(); ++argIt) {
         XQVariable *argVar = new (mm) XQVariable((*argIt)->getURI(), (*argIt)->getName(), mm);
         argVar->setLocationInfo(*argIt);
         newArgs.push_back(argVar);
@@ -415,6 +343,11 @@ void XQUserFunction::staticResolutionStage1(StaticContext *context)
 
 void XQUserFunction::staticResolutionStage2(StaticContext *context)
 {
+  // Prevent static typing being run on this function by a
+  // loop of delayed functions
+  AutoReset<StaticTypingStatus> reset(staticTyped_);
+  staticTyped_ = AFTER;
+
   if(pattern_ != 0 && !pattern_->empty()) {
     VectorOfASTNodes::iterator patIt = pattern_->begin();
     for(; patIt != pattern_->end(); ++patIt) {
@@ -444,10 +377,13 @@ public:
 private:
   virtual ASTNode *optimizeUserFunction(XQUserFunctionInstance *item)
   {
-    // See if we can work out a better return type for the user defined function.
-    // This call will just return if it's already been static typed
-    const_cast<XQUserFunction*>(item->getFunctionDefinition())->staticTypingOnce(context_, styper_);
-
+    XQQuery *module = context_->getModule()->
+      findModuleForFunction(item->getFunctionURI(), item->getFunctionName(), item->getArguments().size());
+    if(module == context_->getModule()) {
+      // See if we can work out a better return type for the user defined function.
+      // This call will just return if it's already been static typed
+      const_cast<XQUserFunction*>(item->getFunctionDefinition())->staticTypingOnce(context_, styper_);
+    }
     return ASTVisitor::optimizeUserFunction(item);
   }
 
@@ -507,17 +443,59 @@ void XQUserFunction::staticTypingOnce(StaticContext *context, StaticTyper *stype
 {
   // Avoid inifinite recursion for recursive functions
   // TBD Need to declare everything as being used - jpcs
-  if(staticTyped_) {
-    recursive_ = true;
+  if(staticTyped_ != BEFORE) {
+    if(staticTyped_ == DURING)
+      recursive_ = true;
+
+    XQGlobalVariable *global = 0;
+    StaticTyper::PrologItem *breadcrumb = styper->getTrail();
+    for(; breadcrumb; breadcrumb = breadcrumb->prev) {
+      if(breadcrumb->global) global = breadcrumb->global;
+      if(breadcrumb->function == this) break;
+    }
+
+    if(global && breadcrumb) {
+      XMLBuffer buf;
+      buf.append(X("The initializing expression for variable {"));
+      buf.append(global->getVariableURI());
+      buf.append(X("}"));
+      buf.append(global->getVariableLocalName());
+      buf.append(X(" depends on itself [err:XQST0054]"));
+      XQThrow3(StaticErrorException, X("XQUserFunction::staticTypingOnce"), buf.getRawBuffer(), global);
+    }
+
     return;
   }
-  staticTyped_ = true;
-  staticTyping(context, styper);
+  staticTyped_ = DURING;
+
+  StaticTyper::PrologItem breadcrumb(this, styper->getTrail());
+  AutoReset<StaticTyper::PrologItem*> autorReset2(styper->getTrail());
+  styper->getTrail() = &breadcrumb;;
+
+  GlobalVariables globalsUsed(XQillaAllocator<XQGlobalVariable*>(context->getMemoryManager()));
+  {
+    AutoReset<GlobalVariables*> autoReset(styper->getGlobalsUsed());
+    styper->getGlobalsUsed() = &globalsUsed;
+    staticTyping(context, styper);
+  }
+
+  if(!globalsUsed.empty()) {
+    // Static type the global variables we depend on
+    GlobalVariables::iterator it = globalsUsed.begin();
+    for(; it != globalsUsed.end(); ++it) {
+      (*it)->staticTypingOnce(context, styper);
+    }
+
+    // Re-static type this function definition
+    staticTyping(context, styper);
+  }
+
+  staticTyped_ = AFTER;
 }
 
 void XQUserFunction::resetStaticTypingOnce()
 {
-  staticTyped_ = false;
+  staticTyped_ = BEFORE;
 }
 
 void XQUserFunction::staticTyping(StaticContext *context, StaticTyper *styper)
@@ -525,7 +503,7 @@ void XQUserFunction::staticTyping(StaticContext *context, StaticTyper *styper)
   // Nothing more to do for external functions
   if(body_ == NULL) return;
 
-  if(isUpdating_ && returnType_ != NULL) {
+  if(signature_->updating == FunctionSignature::OP_TRUE && signature_->returnType != NULL) {
     XQThrow(StaticErrorException, X("XQUserFunction::staticTyping"),
             X("It is a static error for an updating function to declare a return type [err:XUST0028]"));
   }
@@ -555,11 +533,6 @@ void XQUserFunction::staticTyping(StaticContext *context, StaticTyper *styper)
     ciTypeSet = true;
     ciType = StaticType::ITEM_TYPE;
   }
-  if(!isTemplate_ && name_ == 0) {
-    // Inline function
-    ciTypeSet = true;
-    ciType = StaticType::ITEM_TYPE;
-  }
 
   // define the new variables in a new scope and assign them the proper values
   if(context) {
@@ -569,9 +542,9 @@ void XQUserFunction::staticTyping(StaticContext *context, StaticTyper *styper)
     else varStore->addLogicalBlockScope();
 
     // Declare the parameters
-    if(argSpecs_) {
+    if(signature_->argSpecs) {
       ArgumentSpecs::iterator it;
-      for(it = argSpecs_->begin(); it != argSpecs_->end (); ++it) {
+      for(it = signature_->argSpecs->begin(); it != signature_->argSpecs->end (); ++it) {
         varStore->declareVar((*it)->getURI(), (*it)->getName(), (*it)->getStaticAnalysis());
       }
     }
@@ -586,7 +559,7 @@ void XQUserFunction::staticTyping(StaticContext *context, StaticTyper *styper)
   if(context)
     context->getVariableTypeStore()->removeScope();
 
-  if(isUpdating_) {
+  if(signature_->updating == FunctionSignature::OP_TRUE) {
     if(!body_->getStaticAnalysis().isUpdating() && !body_->getStaticAnalysis().isPossiblyUpdating())
       XQThrow(StaticErrorException, X("XQUserFunction::staticTyping"),
               X("It is a static error for the body expression of a user defined updating function "
@@ -609,8 +582,8 @@ void XQUserFunction::staticTyping(StaticContext *context, StaticTyper *styper)
   // Remove the parameter variables from the stored StaticAnalysis
   src_.clear();
   src_.copy(body_->getStaticAnalysis());
-  if(argSpecs_) {
-    for(ArgumentSpecs::iterator it = argSpecs_->begin(); it != argSpecs_->end (); ++it) {
+  if(signature_->argSpecs) {
+    for(ArgumentSpecs::iterator it = signature_->argSpecs->begin(); it != signature_->argSpecs->end (); ++it) {
       if(!src_.removeVariable((*it)->getURI(), (*it)->getName())) {
         // The parameter isn't used, so set it to null, so that we don't bother to evaluate it
         (*it)->setNotUsed();
@@ -626,9 +599,9 @@ void XQUserFunction::staticTyping(StaticContext *context, StaticTyper *styper)
     VariableTypeStore *varStore = context->getVariableTypeStore();
     varStore->addLogicalBlockScope();
 
-    if(argSpecs_ != 0) {
-      XQUserFunction::ArgumentSpecs::const_iterator argIt;
-      for(argIt = argSpecs_->begin(); argIt != argSpecs_->end(); ++argIt) {
+    if(signature_->argSpecs != 0) {
+      ArgumentSpecs::const_iterator argIt;
+      for(argIt = signature_->argSpecs->begin(); argIt != signature_->argSpecs->end(); ++argIt) {
         varStore->declareVar((*argIt)->getURI(), (*argIt)->getName(), templateVarSrc);
       }
     }
@@ -645,21 +618,23 @@ void XQUserFunction::staticTyping(StaticContext *context, StaticTyper *styper)
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
 XQUserFunctionInstance::XQUserFunctionInstance(const XQUserFunction* funcDef, const VectorOfASTNodes& args, XPath2MemoryManager* expr) 
-  : XQFunction(funcDef->getName(), 0, UNLIMITED,"",args, expr),
-    addReturnCheck_(funcDef->body_ == NULL && funcDef->returnType_ != NULL),
+  : XQFunction(funcDef->getName(), 0, args, expr),
+    addReturnCheck_(funcDef->body_ == NULL && funcDef->signature_->returnType != NULL),
     funcDef_(funcDef)
 {
   _type = ASTNode::USER_FUNCTION;
-  _fURI = funcDef->getURI();
+  uri_ = funcDef->getURI();
+  signature_ = funcDef->getSignature();
 }
 
 XQUserFunctionInstance::XQUserFunctionInstance(const XQUserFunction *funcDef, const VectorOfASTNodes& args, bool addReturnCheck, XPath2MemoryManager *mm)
-  : XQFunction(funcDef->getName(), 0, UNLIMITED,"",args, mm),
+  : XQFunction(funcDef->getName(), 0, args, mm),
     addReturnCheck_(addReturnCheck),
     funcDef_(funcDef)
 {
   _type = ASTNode::USER_FUNCTION;
-  _fURI = funcDef->getURI();
+  uri_ = funcDef->getURI();
+  signature_ = funcDef->getSignature();
 }
 
 Result XQUserFunctionInstance::getArgument(size_t index, DynamicContext *context) const
@@ -669,15 +644,6 @@ Result XQUserFunctionInstance::getArgument(size_t index, DynamicContext *context
   return _args[index]->createResult(context);
 }
 
-Result XQUserFunctionInstance::createResult(DynamicContext* context, int flags) const
-{
-  if(funcDef_->body_ != NULL) {
-    return new FunctionEvaluatorResult(this, context);
-  } else {
-    return new ExternalFunctionEvaluatorResult(this);
-  }
-}
-
 ASTNode* XQUserFunctionInstance::staticResolution(StaticContext* context)
 {
   XPath2MemoryManager *mm = context->getMemoryManager();
@@ -685,15 +651,15 @@ ASTNode* XQUserFunctionInstance::staticResolution(StaticContext* context)
   // We don't trust external functions, so check their return type
   if(addReturnCheck_) {
     addReturnCheck_ = false;
-    XQTreatAs *treatAs = new (mm) XQTreatAs(this, funcDef_->returnType_, mm);
-    treatAs->setLocationInfo(funcDef_->returnType_);
+    XQTreatAs *treatAs = new (mm) XQTreatAs(this, funcDef_->signature_->returnType, mm);
+    treatAs->setLocationInfo(funcDef_->signature_->returnType);
     return treatAs->staticResolution(context);
   }
 
-  if(funcDef_->argSpecs_ != 0) {
+  if(funcDef_->signature_->argSpecs != 0) {
     VectorOfASTNodes::iterator argIt = _args.begin();
-    for(XQUserFunction::ArgumentSpecs::iterator defIt = funcDef_->argSpecs_->begin();
-        defIt != funcDef_->argSpecs_->end() && argIt != _args.end(); ++defIt, ++argIt) {
+    for(ArgumentSpecs::iterator defIt = funcDef_->signature_->argSpecs->begin();
+        defIt != funcDef_->signature_->argSpecs->end() && argIt != _args.end(); ++defIt, ++argIt) {
       // The spec doesn't allow us to skip static errors, so we have to check even if
       // the parameter isn't used
       *argIt = (*defIt)->getType()->convertFunctionArg(*argIt, context, /*numericfunction*/false, *argIt);
@@ -742,16 +708,14 @@ ASTNode *XQUserFunctionInstance::staticTypingImpl(StaticContext *context)
 
 void XQUserFunctionInstance::evaluateArguments(VarStoreImpl &scope, DynamicContext *context) const
 {
-  if(funcDef_->getArgumentSpecs() != 0) {
+  if(funcDef_->getSignature()->argSpecs != 0) {
     // the variables should be evaluated in the calling context
     // (before the VariableStore::addLocalScope call: after this call, the variables that can be seen are only the local ones)
     VectorOfASTNodes::const_iterator argIt = _args.begin();
-    for(XQUserFunction::ArgumentSpecs::const_iterator defIt = funcDef_->argSpecs_->begin();
-        defIt != funcDef_->argSpecs_->end() && argIt != _args.end(); ++defIt, ++argIt) {
+    for(ArgumentSpecs::const_iterator defIt = funcDef_->signature_->argSpecs->begin();
+        defIt != funcDef_->signature_->argSpecs->end() && argIt != _args.end(); ++defIt, ++argIt) {
       if((*defIt)->isUsed()) {
-        // TBD ClosureResult doesn't save RegexGroupStore - jpcs
-        // TBD variable use count - jpcs
-        scope.setVar((*defIt)->getURI(), (*defIt)->getName(), ClosureResult::create(*argIt, context));
+        scope.setVar((*defIt)->getURI(), (*defIt)->getName(), (*argIt)->createResult(context));
       }
       else {
         // Skip evaluation of the parameter, since it isn't used, and debugging isn't enabled
@@ -808,74 +772,77 @@ PendingUpdateList XQUserFunctionInstance::createUpdateList(DynamicContext *conte
   return result;
 }
 
-XQUserFunctionInstance::FunctionEvaluatorResult::FunctionEvaluatorResult(const XQUserFunctionInstance *di, DynamicContext *context)
-  : ResultImpl(di),
-    _di(di)
-{
-}
-
-Item::Ptr XQUserFunctionInstance::FunctionEvaluatorResult::nextOrTail(Result &tail, DynamicContext *context)
+Result XQUserFunctionInstance::createResult(DynamicContext* context, int flags) const
 {
   context->testInterrupt();
 
-  VarStoreImpl scope(context->getMemoryManager(), _di->getFunctionDefinition()->isGlobal() ?
-                     context->getGlobalVariableStore() : context->getVariableStore());
-  _di->evaluateArguments(scope, context);
+  if(funcDef_->body_ != NULL) {
+    VarStoreImpl scope(context->getMemoryManager(), funcDef_->isGlobal() ?
+                       context->getGlobalVariableStore() : context->getVariableStore());
+    evaluateArguments(scope, context);
 
-  AutoRegexGroupStoreReset reset3(context);
-  if(!_di->getFunctionDefinition()->isTemplate()) context->setRegexGroupStore(0);
+    AutoRegexGroupStoreReset reset3(context);
+    if(!funcDef_->isTemplate()) context->setRegexGroupStore(0);
 
-  AutoDocumentCacheReset reset(context);
-  DocumentCache* docCache = _di->getFunctionDefinition()->getModuleDocumentCache();
-  if(docCache)
-    context->setDocumentCache(docCache);
+    AutoDocumentCacheReset reset(context);
+    DocumentCache* docCache = funcDef_->getModuleDocumentCache();
+    if(docCache) context->setDocumentCache(docCache);
 
-  tail = ClosureResult::create(_di->getFunctionDefinition()->getFunctionBody(), context, &scope);
-  return 0;
-
-  // TBD Solve this problem again - jpcs
-//   // if we had to switch document cache, check that the returned types are known also in the original context; if not, upgrade them to the base type
-//   if(docCache!=NULL)
-//   {
-//     if(item!=NULLRCP && !reset.oldDC->isTypeDefined(item->getTypeURI(), item->getTypeName()))
-//     {
-//       if(item->isNode())
-//       {
-//         Node::Ptr node = item;
-//         // TODO: change the annotation in the DOM elements and attributes
-//       }
-//       else if(item->isAtomicValue())
-//       {
-//         AnyAtomicType::Ptr atom = item;
-//         const XMLCh* uri = atom->getTypeURI(), *name = atom->getTypeName();
-//         while(!reset.oldDC->isTypeDefined(uri, name))
-//         {
-//             DatatypeValidator* pDV = docCache->getDatatypeValidator(uri, name);
-//             assert(pDV!=NULL);
-//             DatatypeValidator* pBaseDV = pDV->getBaseValidator();
-//             if(pBaseDV==NULL)
-//                 break;
-//             uri = pBaseDV->getTypeUri();
-//             name = pBaseDV->getTypeLocalName();
-//         }
-//         item = context->getItemFactory()->createDerivedFromAtomicType(uri, name, atom->asString(context), context);
-//       }
-//     }
-//   }
-//   return item;
+    AutoVariableStoreReset vsReset(context, &scope);
+    return ClosureResult::create(funcDef_->getFunctionBody(), context);
+  } else {
+    return funcDef_->getExternalFunction()->execute(this, context);
+  }
 }
 
-XQUserFunctionInstance::ExternalFunctionEvaluatorResult::ExternalFunctionEvaluatorResult(const XQUserFunctionInstance *di)
-  : ResultImpl(di),
-    _di(di)
+////////////////////////////////////////////////////////////////////////////////////////////////////
+
+DelayedFuncFactory::DelayedFuncFactory(const XMLCh *uri, const XMLCh *name, size_t numArgs,
+                                       const XMLCh *body, int line, int column, XQQuery *query)
+  : FuncFactory(uri, name, numArgs, numArgs, query->getStaticContext()->getMemoryManager()),
+    body_(body),
+    body8_(0),
+    line_(line),
+    column_(column),
+    query_(query),
+    function_(0)
 {
 }
 
-Item::Ptr XQUserFunctionInstance::ExternalFunctionEvaluatorResult::nextOrTail(Result &tail, DynamicContext *context)
+DelayedFuncFactory::DelayedFuncFactory(const XMLCh *uri, const XMLCh *name, size_t numArgs,
+                                       const char *body, int line, int column, XQQuery *query)
+  : FuncFactory(uri, name, numArgs, numArgs, query->getStaticContext()->getMemoryManager()),
+    body_(0),
+    body8_(body),
+    line_(line),
+    column_(column),
+    query_(query),
+    function_(0)
 {
-  context->testInterrupt();
+}
 
-  tail = _di->getFunctionDefinition()->getExternalFunction()->execute(_di, context);
-  return 0;
+ASTNode *DelayedFuncFactory::createInstance(const VectorOfASTNodes &args, XPath2MemoryManager* memMgr) const
+{
+  if(function_ == 0) {
+    DynamicContext *context = (DynamicContext*)query_->getStaticContext();
+
+    if(body_ == 0) {
+      const_cast<const XMLCh *&>(body_) = context->getMemoryManager()->getPooledString(body8_);
+    }
+
+    XQLexer lexer(memMgr, _LANG_FUNCDECL_, query_->getFile(), line_, column_, body_);
+    XQParserArgs args(&lexer, query_);
+    XQParser::yyparse(&args);
+
+    const_cast<XQUserFunction*&>(function_) = args._function;
+    function_->setDelayed(true);
+
+    query_->addFunction(function_);
+    function_->staticResolutionStage1(context);
+
+    AutoDelete<Optimizer> optimizer(XQilla::createOptimizer(context, XQilla::NO_OPTIMIZATION));
+    optimizer->startOptimize(function_);
+  }
+  return function_->createInstance(args, memMgr);
 }
 
