@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2015 Oracle and/or its affiliates. All rights reserved.
+ *     
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,8 +15,6 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "XQLexer.hpp"
@@ -42,19 +40,25 @@ XQLexer::XQLexer(XPath2MemoryManager* memMgr, const XMLCh *queryFile, const XMLC
 {
   if((m_language & XQilla::XPATH2) != 0) {
     firstToken_ = _LANG_XPATH2_;
-  } else if((m_language & XQilla::FULLTEXT) != 0) {
-    if((m_language & XQilla::UPDATE) != 0) {
-      firstToken_ = _LANG_XQUERY_FULLTEXT_UPDATE_;
-    } else {
-      firstToken_ = _LANG_XQUERY_FULLTEXT_;
-    }
-  } else if((m_language & XQilla::UPDATE) != 0) {
-    firstToken_ = _LANG_XQUERY_UPDATE_;
   } else {
     firstToken_ = _LANG_XQUERY_;
   }
 }
 
+XQLexer::XQLexer(XPath2MemoryManager* memMgr, int firstToken, const XMLCh *queryFile, int line, int column, const XMLCh *query)
+  : Lexer(memMgr, (XQilla::Language)(XQilla::XQUERY3_FULLTEXT_UPDATE | XQilla::EXTENSIONS), queryFile, line, column),
+    firstToken_(firstToken),
+    mode_(MODE_NORMAL),
+    m_szQuery(XPath2Utils::normalizeEOL(query, memMgr)),
+    m_ownQuery(true),
+    m_nLength(XPath2Utils::uintStrlen(m_szQuery)),
+    m_position(0),
+    m_index(0),
+    m_currentOffset(0),
+    m_nOpenComments(0)
+{
+}
+
 XQLexer::XQLexer(XPath2MemoryManager* memMgr, const XMLCh *queryFile, int line, int column, const XMLCh* query,
                  unsigned int length, const std::vector<ValueOffset> &offsets, XQilla::Language lang, StartMode mode)
   : Lexer(memMgr, lang, queryFile, line, column),
@@ -295,7 +299,7 @@ int XQLexer::LexerInput( YY_CHAR* buf, int max_size )
 
   if(nToRead>max_size)
     nToRead=max_size;
-  XMLString::subString((XMLCh*)buf,m_szQuery,m_position,m_position+nToRead);
+  memcpy(buf, m_szQuery + m_position, nToRead * sizeof(XMLCh));
   m_position+=nToRead;
   return nToRead;
 }
