@@ -1,8 +1,8 @@
 /*
- * Copyright (c) 2001-2008
+ * Copyright (c) 2001, 2008,
  *     DecisionSoft Limited. All rights reserved.
- * Copyright (c) 2004-2008
- *     Oracle. All rights reserved.
+ * Copyright (c) 2004, 2015 Oracle and/or its affiliates. All rights reserved.
+ *     
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -15,33 +15,37 @@
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
- *
- * $Id$
  */
 
 #include "../config/xqilla_config.h"
 #include <sstream>
 #include <xqilla/runtime/SequenceResult.hpp>
+#include <xqilla/runtime/Result.hpp>
+#include <xqilla/context/DynamicContext.hpp>
 
 SequenceResult::SequenceResult(const LocationInfo *o, const Sequence &seq)
   : ResultImpl(o),
-    seq_(seq)
+    seq_(seq),
+    i_(0)
 {
-  it_ = seq_.begin();
 }
 
-Item::Ptr SequenceResult::next(DynamicContext *context)
+Item::Ptr SequenceResult::nextOrTail(Result &tail, DynamicContext *context)
 {
-  if(it_ == seq_.end()) {
+  if(i_ >= seq_._itemList.size()) {
+    tail = 0;
     return 0;
   }
-  else {
-    return *(it_++);
-  }
+  return seq_._itemList[i_++];
 }
 
 Sequence SequenceResult::toSequence(DynamicContext *context)
 {
-  return seq_;
+  return Sequence(seq_, i_);
+}
+
+void SequenceResult::skip(unsigned count, DynamicContext *context)
+{
+  i_ += count;
 }
 
