@@ -16,7 +16,7 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  *
- * $Id: runner.cpp 687 2008-11-26 14:25:40Z jpcs $
+ * $Id$
  */
 
 #ifdef _MSC_VER
@@ -33,13 +33,13 @@
 #include <stdio.h>
 
 #include <xercesc/framework/URLInputSource.hpp>
+#include <xercesc/framework/MemBufInputSource.hpp>
 #include <xercesc/util/XMLEntityResolver.hpp>
 #include <xercesc/framework/MemBufFormatTarget.hpp>
 #include <xercesc/dom/DOMException.hpp>
 
 #include <xqilla/xqilla-simple.hpp>
 #include <xqilla/context/VariableStore.hpp>
-#include <xqilla/functions/FunctionConstructor.hpp>
 #include <xqilla/utils/XQillaPlatformUtils.hpp>
 #include <xqilla/events/ContentSequenceFilter.hpp>
 
@@ -51,7 +51,7 @@ using namespace std;
 class XQillaTestSuiteRunner : public TestSuiteRunner, private XMLEntityResolver, private ModuleResolver, private URIResolver
 {
 public:
-  XQillaTestSuiteRunner(const string &singleTest, TestSuiteResultListener *results, XQillaConfiguration *conf, XQilla::Language lang);
+  XQillaTestSuiteRunner(const string &singleTest, TestSuiteResultListener *results, XQillaConfiguration *conf, XQilla::Language lang, FastXDMConfiguration *fastConf = 0, XercesConfiguration *xercesConf = 0, bool userSetParserLang = false);
   virtual ~XQillaTestSuiteRunner();
 
   virtual void addSource(const string &id, const string &filename, const string &schema);
@@ -63,6 +63,7 @@ public:
   virtual void endTestGroup();
 
   virtual void runTestCase(const TestCase &testCase);
+  virtual void detectParserLang(const string &testSuiteNamespace);
 
 private:
   virtual InputSource* resolveEntity(XMLResourceIdentifier* resourceIdentifier);
@@ -72,8 +73,12 @@ private:
   virtual bool resolveDefaultCollection(Sequence &result, DynamicContext* context, const QueryPathNode *projection);
   virtual bool putDocument(const Node::Ptr &document, const XMLCh *uri, DynamicContext *context) { return true; }
 
+  virtual void detectDefaultConf();
 private:
   XQillaConfiguration *m_conf;
+  FastXDMConfiguration *m_fastConf;
+  XercesConfiguration *m_xercesConf;
+  bool m_userSetParserLang;
   XQilla::Language m_lang;
   string m_szSingleTest;
   string m_szFullTestName;
@@ -104,13 +109,15 @@ void usage(const char *progname)
     }
   }
 
-  cout << "Usage: " << name << " [options] <location of the XQTS suite> (<test group or case name>)?" << endl << endl;
+  cout << "Usage: " << name << " [options] <location of the XQTS/XQUTS/XQFTTS suite> (<test group or case name>)?" << endl << endl;
   cout << "-e <file>      : Use the given file as a known error file" << endl;
   cout << "-E <file>      : Output an error file" << endl;
   cout << "-h             : Show this display" << endl;
   cout << "-r             : Output results as XML" << endl;
   cout << "-u             : Parse XQuery Update (also uses Xerces-C data model)" << endl;
   cout << "-s             : Parse XSLT 2.0" << endl;
+  cout << "-f             : Parse XQuery Full Text 1.0" << endl;
+  cout << "-3             : Parse XQuery 3.0" << endl;
   cout << "-x             : Use the Xerces-C data model (default is FastXDM)" << endl;
 }
 
@@ -120,13 +127,14 @@ int main(int argc, char *argv[])
   string singleTest;
   string errorFile;
   string outputErrorFile;
+
   bool xmlResults = false;
-  bool update = false;
-  bool xslt = false;
+  bool userSetParserLang = false;
+  XQilla::Language lang = XQilla::XQUERY;
 
   XercesConfiguration xercesConf;
   FastXDMConfiguration fastConf;
-  XQillaConfiguration *conf = &fastConf;
+  XQillaConfiguration *conf = NULL;
 
   for(int i = 1; i < argc; ++i) {
     if(*argv[i] == '-' && argv[i][2] == '\0' ){
@@ -154,17 +162,29 @@ int main(int argc, char *argv[])
         outputErrorFile = argv[i];
         break;
       }
+      case 'f': {
+        lang = (XQilla::Language)(lang | XQilla::FULLTEXT);
+        userSetParserLang = true;
+        break;
+      }
       case 'r': {
         xmlResults = true;
         break;
       }
       case 'u': {
-        update = true;
+        lang = (XQilla::Language)(lang | XQilla::UPDATE);
+        userSetParserLang = true;
         conf = &xercesConf;
         break;
       }
+      case '3': {
+        lang = (XQilla::Language)(lang | XQilla::VERSION3);
+        userSetParserLang = true;
+        break;
+      }
       case 's': {
-        xslt = true;
+        lang = (XQilla::Language)(lang | XQilla::XSLT2);
+        userSetParserLang = true;
         break;
       }
       case 'x': {
@@ -206,7 +226,7 @@ int main(int argc, char *argv[])
     xmlreport->setImplementation("XQilla", "2.0");
     xmlreport->setOrganization("XQilla", "http://xqilla.sourceforge.net");
 
-    if(!update) {
+    if(lang != XQilla::XQUERY_UPDATE) {
       xmlreport->addImplementationDefinedItem("expressionUnicode", "UTF-16");
       xmlreport->addImplementationDefinedItem("implicitTimezone", "Defined by the system clock");
       xmlreport->addImplementationDefinedItem("XMLVersion", "1.1");
@@ -217,7 +237,7 @@ int main(int argc, char *argv[])
     }
 
     xmlreport->addFeature("Minimal Conformance", true);
-    if(!update) {
+    if(lang != XQilla::XQUERY_UPDATE) {
       xmlreport->addFeature("Schema Import", true);
       xmlreport->addFeature("Schema Validation", true);
       xmlreport->addFeature("Static Typing", false);
@@ -229,7 +249,7 @@ int main(int argc, char *argv[])
     }
 
     xmlreport->setSubmittor("John Snelson", "john.snelson@oracle.com");
-  }
+  } 
   else {
     results.reset(new ConsoleResultListener());
   }
@@ -238,12 +258,8 @@ int main(int argc, char *argv[])
   if(errorFile != "" && !knownErrors.loadErrors(errorFile)) {
     return 1;
   }
-
-  XQilla::Language lang = XQilla::XQUERY;
-  if(update) lang = XQilla::XQUERY_UPDATE;
-  else if(xslt) lang = XQilla::XSLT2;
-
-  XQillaTestSuiteRunner runner(singleTest, &knownErrors, conf, lang);
+  
+  XQillaTestSuiteRunner runner(singleTest, &knownErrors, conf, lang, &fastConf, &xercesConf, userSetParserLang);
   TestSuiteParser parser(testSuitePath, &runner);
 
   parser.run();
@@ -270,9 +286,12 @@ int main(int argc, char *argv[])
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 
-XQillaTestSuiteRunner::XQillaTestSuiteRunner(const string &singleTest, TestSuiteResultListener *results, XQillaConfiguration *conf, XQilla::Language lang)
+XQillaTestSuiteRunner::XQillaTestSuiteRunner(const string &singleTest, TestSuiteResultListener *results, XQillaConfiguration *conf, XQilla::Language lang, FastXDMConfiguration *fastConf, XercesConfiguration *xercesConf, bool userSetParserLang)
   : TestSuiteRunner(results),
     m_conf(conf),
+    m_fastConf(fastConf),
+    m_xercesConf(xercesConf),
+    m_userSetParserLang(userSetParserLang),
     m_lang(lang),
     m_szSingleTest(singleTest),
     m_pCurTestCase(NULL),
@@ -333,6 +352,11 @@ void XQillaTestSuiteRunner::runTestCase(const TestCase &testCase)
     return;
   }
 
+  if(m_results->isSkippedTest(testCase.name)) {
+    m_results->reportSkip(testCase, "Skip");
+    return;
+  }
+
   if(m_szFullTestName.substr(0,21)=="Optional:StaticTyping" && !testCase.updateTest) {
     m_results->reportSkip(testCase, "Static typing not supported");
     return;
@@ -379,8 +403,9 @@ void XQillaTestSuiteRunner::runTestCase(const TestCase &testCase)
 
     map<string, string>::const_iterator v;
     for(v=testCase.extraVars.begin();v!=testCase.extraVars.end();v++) {
-      AutoDelete<XQQuery> pInnerQuery(xqilla.parseFromURI(X(v->second.c_str()), context, XQilla::NO_ADOPT_CONTEXT));
-      Sequence doc=pInnerQuery->execute(context)->toSequence(context);
+      AutoDelete<XQQuery> pInnerQuery(xqilla.parseFromURI(X(v->second.c_str())));
+      AutoDelete<DynamicContext> ictxt(context->createModuleDynamicContext(pInnerQuery->getStaticContext()));
+      Sequence doc=pInnerQuery->execute(ictxt)->toSequence(ictxt);
       context->setExternalVariable(X(v->first.c_str()), doc);
     }
     for(v=testCase.inputVars.begin();v!=testCase.inputVars.end();v++) {
@@ -497,6 +522,13 @@ InputSource* XQillaTestSuiteRunner::resolveEntity(XMLResourceIdentifier* resourc
         }
       }
     }
+
+    // Don't hit the W3C for the XHTML DTD!
+    if(string("http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd") ==
+       UTF8(resourceIdentifier->getSystemId())) {
+      return new MemBufInputSource((XMLByte*)"", 0, "", false);
+    }
+
     return NULL;
 }
 
@@ -546,3 +578,42 @@ bool XQillaTestSuiteRunner::resolveDefaultCollection(Sequence &result, DynamicCo
     return resolveCollection(result, X(m_pCurTestCase->defaultCollection.c_str()), context, projection);
   return false;
 }
+
+void XQillaTestSuiteRunner::detectParserLang(const string &testSuiteNS)
+{
+
+  // if the user specified the parser language, don't do auto-detection 
+  if(m_userSetParserLang){
+    detectDefaultConf();
+    m_lang = (XQilla::Language)(m_lang | XQilla::EXTENSIONS);
+    return; 
+  }
+
+  if(testSuiteNS == "http://www.w3.org/2005/02/query-test-XQTSCatalog")
+     m_lang = XQilla::XQUERY;
+  else if(testSuiteNS == "http://www.w3.org/2005/02/query-test-update")
+     m_lang = XQilla::XQUERY_UPDATE;
+  else if(testSuiteNS == "http://www.w3.org/2005/02/query-test-full-text")
+     m_lang = XQilla::XQUERY_FULLTEXT;
+  else if(testSuiteNS == "http://www.w3.org/2005/05/xslt20-test-catalog")
+     m_lang = XQilla::XSLT2;
+
+  detectDefaultConf();
+
+  m_lang = (XQilla::Language)(m_lang | XQilla::EXTENSIONS);
+}
+
+void XQillaTestSuiteRunner::detectDefaultConf()
+{
+  // if the user has set the m_conf, do nothing
+  if(m_conf != NULL)
+    return;
+
+  if(m_lang & XQilla::UPDATE){
+    m_conf = m_xercesConf;
+  } 
+  else {
+    m_conf = m_fastConf;
+  }
+}
+
